/*  $Header$

    Copyright (c) 1992 APM Software Ltd

    Title:          xmsg
    Author:         Andrew Marlow
    Description:    This module contains the main routine for the xmsg program.
                    This is the Motif version with a push button to 
                    acknowledge the message.
--------------------------------------------------------------------------------
Revision History:

RCS Source History Block:

$Log$
--------------------------------------------------------------------------------
*/

/* Header files */

#include <stdio.h>

#include <X11/Intrinsic.h>
#include <Xm/Xm.h>
#include <Xm/MessageB.h>
#include <Xm/Label.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xresource.h>
#include <X11/cursorfont.h>

#include "portable.h"
#include "xmsg.h"

/* Import variables */

/* Externally defined library functions */

/* Externally defined program functions */

/* Externally visible functions */

/* Internal functions */

static  unsigned int                GetColour ();
static  void                        DisplayUsage ();
static  XFontStruct                 *get_font ();
static  int                         setupGC ();
static  XtCallbackProc              AcknowledgeMessage ();
static  void                        CreateMessagePopup ();
static  int                         xs_str_array_to_xmstr ();

/* Export variables global to the entire application */

/* Variables global to this module only */

typedef struct msg_list
    {
        struct msg_list *NextPtr;
        char buffer [200];
        XmString msgString;
    } MSG_LIST;

static  MSG_LIST                    *HeadPtr = NULL;
static  Display                     *display = NULL;
static  int                         screen_num;
static  Colormap                    colourmap;
static  Window                      root;
static  int                         FontWidth, FontHeight;
static  unsigned int                foreground, background;


/* ------------------------------------------------------------------------ 
Function:       main
Description:    Main entrypoint of xmsg program.
--------------------------------------------------------------------------- */
GLOBAL void main (argc, argv)
int argc;
char *argv[];
{
    XrmValue value;
    char *str_type [20], MyDisplayName[256], MyFontName [256], buffer[20];
    char fgName [256], bgName [256];
    Window w, root_window, child_window;
    int RootX, RootY, WinX, WinY, ntries, nlines, i;
    unsigned int KeysButtons, width, height, border, border_width;
    XEvent event;
    Pixmap icon_pixmap;
    XSizeHints size_hints;
    XWMHints wm_hints;
    XClassHint class_hints;
    XTextProperty windowName, iconName;
    char text[200];
    char *msg = NULL;
    GC gc;
    XFontStruct *font_info;
    MSG_LIST *ItemPtr, *PrevPtr;
    XtAppContext app_ctx;
    Widget shell;
        
    static char *window_name = "X Windows message";
    static char *icon_name = "xmsg";

    MyDisplayName[0] = '\0';
    strcpy (fgName, "black");
    strcpy (bgName, "white");
    strcpy (MyFontName, "9x15");

    for (i = 1; i < argc; i++)
    {
        if (strcmp (argv[i], "-msg") == 0)
        {
            msg = argv[i+1];
            i++;
        }
        else if ((strcmp (argv[i], "-background") == 0) ||
                 strcmp (argv[i], "-bg") == 0)
        {
            if ((i+1) < argc)
            {
                strcpy (bgName, argv[i+1]);
                i++;
            }
        }
        else if ((strcmp (argv[i], "-foreground") == 0) ||
                 strcmp (argv[i], "-fg") == 0)
        {
            if ((i+1) < argc)
            {
                strcpy (fgName, argv[i+1]);
                i++;
            }
        }
        else if ((strcmp (argv[i], "-font") == 0) ||
                 strcmp (argv[i], "-fn") == 0)
        {
            if ((i+1) < argc)
            {
                strcpy (MyFontName, argv[i+1]);
                i++;
            }
        }
        else if (strcmp (argv[i], "-display") == 0)
        {
            if ((i+1) < argc)
            {
                strcpy (MyDisplayName, argv[i+1]);
                i++;
            }
        }
        else if (strcmp (argv[i], "-help") == 0)
        {
            DisplayUsage ();
            exit (0);
        }
        else
        {
            fprintf (stderr, "Error: Option '%s' not recognised.\n", argv[i]);
            DisplayUsage ();
            exit (0);            
        }
    }

    if (msg == NULL)
    {
        nlines = 0;
        width = 0;
        PrevPtr = NULL;
        while (gets(text))
        {
            nlines++;
            ItemPtr = (MSG_LIST *) malloc (sizeof(MSG_LIST));
            if (ItemPtr == NULL)
            {
                fprintf (stderr, "malloc failed\n");
                exit (1);
            }
            ItemPtr->NextPtr = NULL;
            strcpy (ItemPtr->buffer, text);
            if (strlen(text) > width)
                width = strlen(text);
            if (PrevPtr == NULL)
                HeadPtr = ItemPtr;
            else
                PrevPtr->NextPtr = ItemPtr;
            PrevPtr = ItemPtr;
        }
    }
    else
    {
        HeadPtr = (MSG_LIST *) malloc (sizeof(MSG_LIST));
        if (HeadPtr == NULL)
        {
            fprintf (stderr, "malloc failed\n");
            exit (1);
        }

        nlines = 1;
        HeadPtr->NextPtr = NULL;
        strcpy (HeadPtr->buffer, msg);
        width = strlen(msg);
    }

    if (nlines == 0)
    {
        exit (0);
    }

    /* Initialise the intrinsics */
    
    XtToolkitInitialize ();

    app_ctx = XtCreateApplicationContext ();
    
    /* Try to connect to display (may try a few times) */

    for (display = NULL, ntries = 0; display == NULL && ntries < 5; ntries++)
    {
        display = XtOpenDisplay (app_ctx, MyDisplayName, "xmsg", "Xmsg",
                                 (XrmOptionDescRec *) NULL, (Cardinal) 0, 
                                 &argc, argv);
        if (display == NULL && ntries <= 4)
        {
            printf ("Trying to connect to '%s' ...\n", MyDisplayName);
        }
    }

    if (display == NULL)
    {
        fprintf (stderr, "%s: Cannot open display '%s'\n", argv[0],
                 XDisplayName(MyDisplayName));
        exit (1);
    }

    screen_num = DefaultScreen (display);
    colourmap = DefaultColormap (display, screen_num);

    foreground = GetColour (fgName, TRUE);
    background = GetColour (bgName, FALSE);

    if ((font_info = get_font (MyFontName)) == NULL)
    {
        fprintf (stderr, "Error: Cannot find font '%s'\n", MyFontName);
        exit (1);
    }

    FontWidth = font_info->max_bounds.width;
    width = (width + 3) * FontWidth;
    FontHeight = font_info->ascent + font_info->descent;

    root = RootWindow (display, screen_num);

    /* Find where the cursor is in the root window */

    XQueryPointer (display, root, &root_window, &child_window,
                   &RootX, &RootY, &WinX, &WinY, &KeysButtons);

    border_width = 5;
    border = WhitePixel (display, screen_num);

    height = (nlines + 1) * FontHeight;

    shell = XtAppCreateShell ("xmsg", "Xmsg", applicationShellWidgetClass,
                              display, NULL, 0);

    CreateMessagePopup (shell, MyFontName);
    
    XtRealizeWidget (shell);
    
    XtAppMainLoop (app_ctx);
}


/* ------------------------------------------------------------------------
Function:       CreateMessagePopup
Description:    This routine creates a popup with multiple lines of text
                and an Acknowledge button.
--------------------------------------------------------------------------- */
static void CreateMessagePopup (Widget shell, char *MyFontName)
{
    MSG_LIST *ItemPtr;
    Arg *ArgItem;
    Arg *ArgPtr;
    Cardinal n;
    Widget msg_box, ok_btn, cancel_btn, help_btn;
    int count;
    XmString string;        
    Font font;
    XFontStruct *font_info;
    XmFontList my_fontlist;

    for (count = 0, ItemPtr = HeadPtr; ItemPtr != NULL; ItemPtr = ItemPtr->NextPtr)
        count++;
        
    ArgPtr = (Arg *) malloc (count * 4 * sizeof(Arg));
    if (ArgPtr == NULL)
    {
        fprintf (stderr, "malloc args failed\n");
        exit (1);
    }
    
    font = XLoadFont (display, MyFontName);
    font_info = XQueryFont (XtDisplay(shell), font);
    my_fontlist = XmFontListCreate (font_info, XmSTRING_DEFAULT_CHARSET);

    ArgItem = ArgPtr;
    for (ItemPtr = HeadPtr; ItemPtr != NULL; ItemPtr = ItemPtr->NextPtr)
    {
        ItemPtr->msgString = XmStringCreateLtoR (ItemPtr->buffer, 
                                    XmSTRING_DEFAULT_CHARSET);

        XtSetArg (*ArgItem, XmNmessageString, ItemPtr->msgString);
        ArgItem++;

        XtSetArg (*ArgItem, XmNdialogType, XmDIALOG_MESSAGE);
        ArgItem++;

        XtSetArg (*ArgItem, XmNmessageAlignment, XmALIGNMENT_BEGINNING);
        ArgItem++;

        XtSetArg (*ArgItem, XmNlabelFontList, my_fontlist);
        ArgItem++;
    }

    msg_box = XmCreateMessageBox (shell, "message", ArgPtr, 4);
/*
 count*4-1);
*/

    for (ItemPtr = HeadPtr; ItemPtr != NULL; ItemPtr = ItemPtr->NextPtr)
        XmStringFree (ItemPtr->msgString);
        
    ok_btn = XmMessageBoxGetChild (msg_box, XmDIALOG_OK_BUTTON);
    cancel_btn = XmMessageBoxGetChild (msg_box, XmDIALOG_CANCEL_BUTTON);
    help_btn = XmMessageBoxGetChild (msg_box, XmDIALOG_HELP_BUTTON);
    
    XtUnmanageChild (cancel_btn);
    XtUnmanageChild (help_btn);
    
    /* Change ok button to read "Acknowledge" */
    
    string = XmStringCreateLtoR ("Acknowledge", XmSTRING_DEFAULT_CHARSET);
    
    XtVaSetValues (ok_btn, XmNlabelString, string, NULL);

    /* Add callback to ok button */
    
    XtAddCallback (ok_btn, XmNactivateCallback, AcknowledgeMessage, NULL);
        
    XtManageChild (msg_box);
}

/* ARGSUSED */
static XtCallbackProc
AcknowledgeMessage (widget, recipient, call_data)
    Widget widget;
    char *recipient;
    XtPointer call_data;
{
    exit (0);
}


/* ------------------------------------------------------------------------
Function:       CreateMessagePopup
Description:    This routine creates a popup with multiple lines of text
                and an Acknowledge button.
--------------------------------------------------------------------------- */
static int xs_str_array_to_xmstr ()
{
    return 0;
}

#ifdef GUNK
    w = XCreateSimpleWindow (display, root, RootX, RootY, width, height,
                             border_width, border, background);

    icon_pixmap = XCreateBitmapFromData (display, w, xmsg_bits,
                                         xmsg_width, xmsg_height);

    /* Set size hints for window manager */

    size_hints.flags = PPosition | PSize | PMinSize;
    size_hints.min_width = width;
    size_hints.min_height = height;

    if (XStringListToTextProperty (&window_name, 1, &windowName) == 0)
    {
        fprintf (stderr, "Error: XStringListToTextProperty failed\n");
        exit (1);
    }

    if (XStringListToTextProperty (&icon_name, 1, &iconName) == 0)
    {
        fprintf (stderr, "Error: XStringListToTextProperty failed\n");
        exit (1);
    }

    wm_hints.initial_state = NormalState;
    wm_hints.input = True;
    wm_hints.icon_pixmap = icon_pixmap;
    wm_hints.flags = StateHint | IconPixmapHint | InputHint;

    class_hints.res_name = "xmsg";
    class_hints.res_class = "Xmsg";

    XSetWMProperties (display, w, &windowName, &iconName, argv, argc,
                      &size_hints, &wm_hints, &class_hints);

    XSelectInput (display, w, ExposureMask | ButtonPressMask);

    setupGC (w, &gc, font_info);

    XMapWindow (display, w);

    for (;;)
    {
        XNextEvent (display, &event);
        switch (event.type)
        {
            case Expose:
                i = 0;
                for (ItemPtr = HeadPtr; ItemPtr != NULL; ItemPtr = ItemPtr->NextPtr)
                {
                    i++;
                    XDrawImageString (display, w, gc, FontWidth, 
                                      FontHeight*i,
                                    ItemPtr->buffer, strlen(ItemPtr->buffer));
                }
                break;

            case ButtonPress:
                XUnloadFont (display, font_info->fid);
                XFreeGC (display, gc);
                XCloseDisplay (display);
                exit (0);
                break;

            default:
                break;
        }
    }
}
#endif


static unsigned int GetColour (str, fg)
char *str;
int fg;
{
    XrmValue value;
    XColor rgb_def;
    int ReturnedColour = 0;

    if (XParseColor (display, colourmap, str, &rgb_def) == 0)
    {
        fprintf (stderr, "Warning: Unknown colour: %s\n", str);
        if (fg)
            ReturnedColour = BlackPixel (display, screen_num);
        else
            ReturnedColour = WhitePixel (display, screen_num);
    }
    else
    {
        if (XAllocColor (display, colourmap, &rgb_def) == 0)
        {
            if (fg)
                ReturnedColour = BlackPixel (display, screen_num);
            else
                ReturnedColour = WhitePixel (display, screen_num);
            fprintf (stderr, "Warning: Could not allocate colour %s\n", str);
        }
        else
        {
            ReturnedColour = rgb_def.pixel;
        }
    }
    return (ReturnedColour);
}


static  XFontStruct *get_font (MyFontName)
char *MyFontName;
{
    Font font;
    XFontStruct *font_info;

    font = XLoadFont (display, MyFontName);
    font_info = XQueryFont (display, font);
    return (font_info);
}


static int setupGC (win, gc, font_info)
Window win;
GC *gc;
XFontStruct *font_info;
{
    unsigned long valuemask = 0;
    XGCValues values;
    unsigned int line_width = 6;
    int line_style = LineSolid;
    int cap_style = CapRound;
    int join_style = JoinRound;
    int status = 0;

    *gc = XCreateGC (display, win, valuemask, &values);

    XSetFont (display, *gc, font_info->fid);

    XSetLineAttributes (display, *gc, line_width, line_style, 
                        cap_style, join_style);

    XSetForeground (display, *gc, foreground);
    XSetBackground (display, *gc, background);

    return (status);
}


static void DisplayUsage ()
{
    fprintf (stderr, "Usage: xmsg -msg <text> [Xoptions]\n");
    fprintf (stderr, "\n");
    fprintf (stderr, "\tWhere the X options are as follows:-\n\n");
    fprintf (stderr, "\t\t-background <colour>\n");
    fprintf (stderr, "\t\t-bg <colour>\n");
    fprintf (stderr, "\t\t-foreground <colour>\n");
    fprintf (stderr, "\t\t-fg <colour>\n");
    fprintf (stderr, "\t\t-display <displayname>\n");
    fprintf (stderr, "\t\t-font <fontname>\n");
    fprintf (stderr, "\t\t-fn <fontname>\n");
    fprintf (stderr, "\n");
}

regcmp() {}
regex() {}
/* ------------------------------------------------------------------------ */
